package clothing_search_engine.crawler;

import java.io.*;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Sara
 *
 */
public class PageExtractor {

	private String range;

	/**
	 * Constructor
	 * 
	 * @param m_range regular expression of range of page 
	 */
	public PageExtractor(String m_range) {
		range = m_range;
	}

	/**
	 * Return the entire webpage into one string
	 * 
	 * @param url url of webpage to get
	 * @return String unprocessed webpage
	 */
	public String getPage(URL url) {

		String inputLine;
		String webPage = "";

		try {
			BufferedReader in = new BufferedReader(
					new InputStreamReader(
							url.openStream()));
			// Don't get such large pages to avoid crashing (they are usually irrelevant pages)
			while ((inputLine = in.readLine()) != null) {
				inputLine = inputLine.trim();

				if (inputLine.length() > 0) {
					// To seperate lines without \n for easy search; changed back later
					webPage += inputLine + "--newline here--";
				}
			}
			
			in.close();
		}catch(Exception e) {
			System.out.println("Failure in Crawler.getPage(), Read: " + e);
		}
		
		if (!range.isEmpty()) {
			try {
				Matcher m_range = Pattern.compile(range).matcher(webPage);
				if(m_range.find()) {
					webPage = webPage.substring(m_range.start(),m_range.end());
				}
			}
			catch (Exception e) {
				System.out.println("Failure in Crawler.getPage(), Range: " + e);
			}
		}
		
		webPage = webPage.replaceAll("--newline here--", "\n");
		
		return webPage;
	}
	
	/**
	 * Clean the page to make it useful for search engine analysis
	 * 
	 * @param webPage unprocessed webpage
	 * @return String processed/cleaned webpage
	 */
	public String cleanPage(String webPage) {
	
		String [] webPage_byline = webPage.split("\n");
		String newWebPage = "";
		
		for (int index = 0; index < webPage_byline.length; index++) {
			//skip all javascript
			if(webPage_byline[index].matches(".*script.*javascript.*") || webPage_byline[index].indexOf("<script>") > -1) {
				index++;
				while(index < webPage_byline.length && webPage_byline[index].indexOf("</script>") < 0) {
					index++;
				}
				index++;
			}
			
			//Keep titles and anything not in html tags
			else if(webPage_byline[index].indexOf("title") > -1) {
				int start = webPage_byline[index].indexOf("title");
				int end = webPage_byline[index].indexOf("\"",start+7);
				if (end > 0) {
					webPage_byline[index] = webPage_byline[index].substring(start+7,end);
					//System.out.println(webPage_byline[index]);
					newWebPage += webPage_byline[index] + "\n";
				}
			}			
			else {
				webPage_byline[index] = webPage_byline[index].replaceAll("\\<.*?>","");
				if (webPage_byline[index].trim().length() > 0) {
					newWebPage += webPage_byline[index] + "\n";
				}
			}
		}
		newWebPage = newWebPage.replaceAll("\n","--newline here--");
		newWebPage = newWebPage.replaceAll("\\<.*?>","");
		newWebPage = newWebPage.replaceAll("[\\>\\<]","");
		newWebPage = newWebPage.replaceAll("&nbsp;"," ");
		newWebPage = newWebPage.replaceAll("&gt;",">");
		newWebPage = newWebPage.replaceAll("&lt;","<");
		newWebPage = newWebPage.replaceAll("--newline here--","\n");
		return newWebPage;
		
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		if (args.length < 3) {
			System.out.println("Missing Arguments. Usage: id url_file/url range output_directory");
			return;
		}
		
		String site_id = args[0];
		String url_file = args[1];
		String range = args[2];
		String output_directory = args[3];
		
		//System.out.println("The file/url you entered is : " + url_file);

		PageExtractor page_extractor = new PageExtractor(range);

		try {
			
			//accept one url or a file with a list of urls
			if (url_file.indexOf("www.") >= 0) {
				
				URL url = new URL(url_file);
				
				String webPage = page_extractor.getPage(url);
				webPage = page_extractor.cleanPage(webPage);

				try{
				    // Create file
					String id = site_id + url.hashCode();
				    
					FileWriter fstream = new FileWriter(output_directory + "/" + id + ".txt");
				    BufferedWriter out = new BufferedWriter(fstream);
			        
				    out.write("<id>" + id + "</id>\n");
			        out.write("<url>" + url.toString() + "</url>\n");
			        out.write("<content>");
			        out.write(webPage);
			        out.write("</content>");

			        //Close the output stream
				    out.close();
				}catch (Exception e){//Catch exception if any
				   System.err.println("Write Error: " + e.getMessage());
			    }

			}
			else {
				BufferedReader input = new BufferedReader(new FileReader(url_file));
				String single_url = input.readLine();

				while (single_url != null) {

					URL url = new URL(single_url);
					
					String webPage = page_extractor.getPage(url);
					webPage = page_extractor.cleanPage(webPage);
					
					try{
					    // Create file
						String id = site_id + url.hashCode();
					    
						FileWriter fstream = new FileWriter(output_directory + "/" + id + ".txt");
					    BufferedWriter out = new BufferedWriter(fstream);
				        
					    out.write("<id>" + id + "</id>\n");
				        out.write("<url>" + url.toString() + "</url>\n");
				        out.write("<content>");
				        out.write(webPage);
				        out.write("</content>");
	
				        //Close the output stream
					    out.close();
					}catch (Exception e){//Catch exception if any
					   System.err.println("Write Error: " + e.getMessage());
				    }
					
					single_url = input.readLine();
				}
				input.close();
			}
		}catch(IOException e) {
			System.out.println("Failure in main: " + e);
		}

	}
}
